import javax.swing.*;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * Set up the window and show four buttons and a label.
 * The size of the window is determined from the size of the label.
 * Three of the four buttons are given a "listener", an instance of the new class ButtonHandler.
 * The "west" button doesn't do anything, because it doesn't have a listener.
 * New feature: instead of having ButtonHandler be a separate class, it has become an "inner class"
 * defined inside SetUpOperation.  One advantage of this is that it automatically has access to
 * SetUpOperation's instance variables, so to take advantage of this, I've moved centerText from
 * being a local variable of "run()" to being an instance variable of "SetUpOperation".
 * @version 13, 2 Feb 2005.
 **/
public class SetUpOperation implements Runnable
{
    JLabel centerText = new JLabel ("some text in the center");

    public void run () {
        // Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);
        
        // Create the window and set its size.
        JFrame frame = new JFrame ("My example program");
        
        // Make sure the program exits when the window is closed.
        frame.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
        
        // Add something to the window.
        JButton northButton = new JButton ("north");
        JButton southButton = new JButton ("south");
        JButton eastButton = new JButton ("east");
        JButton westButton = new JButton ("west");
        frame.getContentPane().add(northButton, BorderLayout.NORTH);
        frame.getContentPane().add(centerText, BorderLayout.CENTER);
        frame.getContentPane().add(eastButton, BorderLayout.EAST);
        frame.getContentPane().add(southButton, BorderLayout.SOUTH);
        frame.getContentPane().add(westButton, BorderLayout.WEST);
        
        northButton.addActionListener (new ButtonHandler ("Top button pressed!"));
        southButton.addActionListener (new ButtonHandler ("Bottom button pressed!"));
        eastButton.addActionListener (new ButtonHandler ("Right-hand button pressed!"));
        // Let's leave westButton without a listener for now.
        
        // Compute the size of the window, just big enough to hold the labels.
        frame.pack();
        
        // Show the window.
        frame.setVisible(true);
    }
    
    /**
     * An inner class to listen for button-clicks.
     * @version 13, 3 Feb 2005
     */
    public class ButtonHandler implements ActionListener
    {
        private String newText;
        
        public ButtonHandler (String newText) {
            this.newText = newText;
        }
        
        public void actionPerformed (ActionEvent e) {
            centerText.setText (newText);
            // Note that centerText actually belongs to SetUpOperation.
        }
    }
}